@@ -23,7 +23,7 @@
 ;;;  OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 ;;;  IN THE SOFTWARE.
 ;;;
-;;; $Id: format.scm,v 1.34 2004/04/02 12:10:56 shirok Exp $
+;;; $Id: format.scm,v 1.38 2005/05/28 02:48:00 shirok Exp $
 
 (define-module wiliki.format
   (use srfi-1)
@@ -238,7 +238,7 @@
   ;; NB: the precedence is embedded to the order of calling regexp-fold.
   (define (mailto line seed)
     (regexp-fold
-     #/\[(mailto:[-\w]+(?:\.[-\w]+)*@[-\w]+(?:\.[-\w]+)+)\s+(.*)\]/
+     #/\[(mailto:[-\w]+(?:\.[-\w]+)*@[-\w]+(?:\.[-\w]+)+|telnet:\/\/[-\w.]+(?::\d+)?)\s+(.+)\]/
      cons
      (lambda (match seed)
        (cons `(a (@ (href ,(match 1))) ,(match 2)) seed))
@@ -296,9 +296,13 @@
           (receive (wikiname rest) (find-closer post 0 '())
             (if wikiname
               (bracket rest
-                       (append (reverse! (parameterize ((fmt-context ctx))
-                                           (fmt-wikiname wikiname)))
-                               (bold pre seed)))
+                       (append-reverse!
+                        (parameterize ((fmt-context (filter (lambda (e)
+                                                              (and (pair? e)
+                                                                   (memq (car e) '(h1 h2 h3 h4 h5 h6))))
+                                                            ctx)))
+                          (fmt-wikiname wikiname))
+                        (bold pre seed)))
               (bold rest (bold pre seed))))
           (bold line seed)))))
   (define (smacro line seed)
@@ -550,7 +554,7 @@
              (values tok (finish))))
           (else
            (loop (next-token ctx) '()
-                 (fmt-line ctx (token-value tok) p) r))
+                 (fmt-line ctx (token-value tok) (fold-p))))
           ))))
 
   ;; Main body
